package dmch;

/* RSA.java
 * Matthew Wronka <wronkm@rpi.edu>
 *
 * Some help with using BIGINT, not to mention the algo, from Paj's code at
 * http://pajhome.org.uk/crypt/rsa/
 *
 * Copyright (C) 2002 Matthew Wronka <wronkm@rpi.edu>
 * This file is released under the GNU General Public License version 2,
 * a version of which should have been provided along with this file.
 *
 * You might also find a copy at:
 * http://dmc.sf.net/files/gpl.txt
 * http://www.fsf.org/copyleft/gpl.html
 */

import java.math.BigInteger;

/**
 * Implementation of the Rivest, Shamir, Adleman algorithm for
 * 	asymmetric cryptography.
 *
 * @author Matthew Wronka <wronkm@rpi.edu>
 * @version 2002-11-25
 */
public class RSA {
	private BigInteger n, d, e;

	/**
	 * Generates private/public key pair of lenth <em>bitlen</em>.
	 */
	public RSA(int bitlen){
		java.security.SecureRandom random = new java.security.SecureRandom();
		BigInteger m,
			p=new BigInteger(bitlen/2, 100, random), 
			q=new BigInteger(bitlen/2, 100, random);
		n = p.multiply(q);

		m = (p.subtract(BigInteger.ONE)).multiply(q.subtract(BigInteger.ONE));
		e = new BigInteger(bitlen/4, 100, random);
		while(m.gcd(e).intValue() > 1) e = e.add(new BigInteger("2"));
		d = e.modInverse(m);
	}

	/**
	 * Encrypts <em>msg</em> using our public key.
	 *
	 * @param msg Cleartext to encrypt.
	 * @return Encrypted version of <em>msg</em>
	 *	 as a hextbin string.
	 */
	public String encrypt(BigInteger msg){
		return msg.modPow(e, n).toString(16);
	}

	/**
	 * Decrypts <em>msg</em> using our private key.
	 *
	 * @param msg Ciphertext to be decrypted.
	 * @return Clear text version of <em>msg</em>
	 *	 as a hextbin string.
	 */
	public String decrypt(BigInteger msg){
		return msg.modPow(d, n).toString(16);
	}

        /**
         * Signs <em>msg</em> using our private key.
         *
         * @param msg Cleartext to signed.
         * @return Signed version of <em>msg</em>
	 *	 as a hextbin string.
         */
	public String sign(BigInteger msg) { return decrypt(msg); }


	/**
	 * Returns this public key in the form "e n".
	 *
	 * @return Our public key in the form "e n" as a hextbin string.
	 */
	public String getPublicKey(){
		return e.toString(16) + " " + n.toString(16);
	}

	/**
	 * Overloaded version of encrypt to encrypt with an arbitrary
	 * 	public key in the format "e n", such as those produced
	 * 	by getPublicKey().
	 *
	 * @param msg Cleartext to be encrypted.
	 * @param publicKey Public key used  to encrypt, in the form
	 *	"e n" as a hexbin string.
	 * @return Cphertext version of <em>msg</em> as a hexbin string.
	 */
	public String encrypt(BigInteger msg, String publicKey){
		BigInteger oe, on;
		java.util.StringTokenizer st=new java.util.StringTokenizer(publicKey);
		try {
			oe=new BigInteger(st.nextToken(), 16);	
			on=new BigInteger(st.nextToken(),16);
			return msg.modPow(oe, on).toString(16);
		} catch ( NumberFormatException nfe) {
			return "";	// XXX: be more helpful?
		}
	}
}
