package dmch;

/*
 * UserList.java
 * Matthew Wronka <wronkm@rpi.edu>
 *
 * Manages user (buddy) list.
 *
 * Copyright (C) 2002 Matthew Wronka <wronkm@rpi.edu>
 * This file is released under the GNU General Public License version 2,
 * a version of which should have been provided along with this file.
 *
 * You might also find a copy at:
 * http://dmc.sf.net/files/gpl.txt
 * http://www.fsf.org/copyleft/gpl.html
 */

import javax.swing.*;
import javax.swing.tree.*;

import java.awt.*;
import java.awt.event.*;

import java.util.Enumeration;
import java.io.IOException;

/**
 * Manages user lists over multiple connections.  It uses a tree hierarchy
 * 	with a users connected to groups, connected to connections, 
 *	connected to a single root node.
 *
 * @author Matthew Wronka <wronkm@rpi.edu> 
 * @version 0
 */
public class UserList implements wtalk.IUserList{
	final DefaultMutableTreeNode mTop = new DefaultMutableTreeNode("DMC");
	final DefaultTreeModel treeModel = new DefaultTreeModel(mTop);
	final JTree tree = new JTree(treeModel);
	final JScrollPane  treeView = new JScrollPane(tree);
	TabbedConnection mTC=null;

	/**
	 * Default ctor, creates the tree representation of user lists.
	 *
	 */
	public UserList() {
		createTree();
	}

	/**
	 * Returns the GUI representation of the user list.
	 *
	 * @return GUI element representing this UserList.
	 */
	public JScrollPane getUI() {
		return treeView;
	}

	/**
	 * Adds a node <em>child</em> to <em>parent</em> node.
	 *
	 * @param child Node to be added.
	 * @param parent Node to add child to.
	 */
	protected void addNode(DefaultMutableTreeNode child, DefaultMutableTreeNode parent) {
		treeModel.insertNodeInto(child, parent, parent.getChildCount());
	}

	/**
	 * House keeping to be done after tree manipuplations, such as
	 * 	additions and deletions.
	 *
	 * @param node Node for whom this function is called (or its parent).
	 */
	protected void updateTree(DefaultMutableTreeNode node) {
		//treeModel.reload();
		tree.scrollPathToVisible(new TreePath(node));
	}

	///////
	//
	//IUserList functions
	//
        /**
         * Adds a connection named <em>connectionName</em>.
         * If the connection already exists, no change is made.
         * <p>
         * If there is already a connection by this name,
         *      there is no change made.
         *
         * @param connectionName The name of connection to be added.
         */
	public void addConnection(String connectionName) {
		DefaultMutableTreeNode  newNode=null;

		// add new connection under root
		if(getChild(connectionName, mTop)!=null) return;
		System.err.println("Adding " + connectionName + " to " + mTop);

		newNode = new DefaultMutableTreeNode(connectionName);

		//mTop.add(newNode);
		addNode(newNode, mTop);
		updateTree(newNode);
	}

        /**
         * Adds a group named <em>groupName</em> to <em>connectionName</em>.
         * If <em>groupName</em> already exists
         *      under <em>connectionName</em>, no change is made.
         * If <em>connectionName</em> does not exist, no change is made.
         * <p>
         * If the specified group already exists, no change is made.
         *
         * @param groupName The group to be added.
         * @param connectionName An existing connection to which group
         *      is to be added.
         */
	public void addGroup(String groupName, String connectionName) {
		DefaultMutableTreeNode  newNode=null;

		// add new group under connection name
		DefaultMutableTreeNode conn=getChild(connectionName, mTop);
		if(conn==null) return;

		if(getChild(groupName, conn)!=null) return; // already exists;

		newNode = new DefaultMutableTreeNode(groupName);
		//conn.add(newNode);
		addNode(newNode, conn);
		updateTree(newNode);
	}

        /**
         * Adds <em>userName</em> to the specified group.
         * If the user already exists in the group,
         *      or the group does not exist under the given connection,
         *      no change is made.
         * <p>
         * If the specified user already exists, no change is made.
         *
         * @param userName The name of the user to add.
         * @param groupName The name of the group to add <em>userName</em> to.
         * @param connectionName The connection in which to search for
         *      groupName.
         */
	public void addUser(String userName, String groupName, String connectionName) {
		DefaultMutableTreeNode  newNode=null;

		// add new user under (group under connection)
		DefaultMutableTreeNode conn=getChild(connectionName, mTop);
		DefaultMutableTreeNode grup=conn==null?null:getChild(groupName, conn);
		if(grup==null) return;

		if(getChild(userName, grup)!=null) return; // already exists;

		newNode = new DefaultMutableTreeNode(userName);
		//grup.add(newNode ); 
		addNode(newNode, grup);
		updateTree(newNode);
	}


        /**
         * Removes the named connection, and everything under it.
         * No change is made if <em>connectionName</em> does not exist.
         *
         * @param connectionName Name of connection to remove.
         */
	public void delConnection(String connectionName) {
		// remove connection 
		DefaultMutableTreeNode conn;
		if((conn=getChild(connectionName, mTop))==null) return;
		
		//conn.removeFromParent();
		treeModel.removeNodeFromParent(conn);
		updateTree(mTop);
	}

        /**
         * Removes the named group, and all users under it.
         * No change is made if either the group or connection named
         *      do not exist.
         *
         * @param groupName Name of group to be removed.
         * @param connectionName Name of connection under which group exists.
         */
	public void delGroup(String groupName, String connectionName) {
		// remove group from connection
		DefaultMutableTreeNode conn=getChild(connectionName, mTop);
		DefaultMutableTreeNode grup=conn==null?null:getChild(groupName, conn);
		if(grup==null) return;

		//grup.removeFromParent();
		treeModel.removeNodeFromParent(grup);
		updateTree(conn);
	}

        /**
         * Removes the named user, assuming user exists under the named
         *      group and connection. There should be no change otherwise.
         *
         * @param userName User to be removed from the user list.
         * @param groupName Group name to which User belongs.
         * @param connectionName Connection under which to search
         *      for <em>groupName</em>.
         */
	public void delUser(String userName, String groupName, String connectionName) {
		// remove user from (group under connection)
		DefaultMutableTreeNode conn=getChild(connectionName, mTop);
		DefaultMutableTreeNode grup=(conn==null)?null:getChild(groupName, conn);
		DefaultMutableTreeNode usr=(grup==null)?null:getChild(userName, grup);
		if(usr==null) return;

		//usr.removeFromParent();
		treeModel.removeNodeFromParent(usr);
		updateTree(grup);
	}
	//
	// End IUserList
	//
	///////

	/**
	 * Returns a child node <em>name</em> of <em>node</em>, or null if
	 *	it does not exist.
	 *
	 * @param name Name of child to search for.
	 * @param node Ancestor of <em>name</em>.
	 * @return The descendent of <em>node</em> with name <em>name</em> 
	 *	if it exists; otherwise null.
	 */
	private DefaultMutableTreeNode getChild(String name, DefaultMutableTreeNode node) {
		DefaultMutableTreeNode rval=null;
		for (Enumeration e = node.children(); e.hasMoreElements(); ) 
			if((rval=(DefaultMutableTreeNode)(e.nextElement())).toString().equals(name)) return rval;
		return null;
	}


	/**
	 * Initializes the GUI components used for the graphical representation
	 * of this UserList.
	 *
	 * @return GUI representation of this UserList.
	 */ 
	private Component createTree() {
		final DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer();
		// Initialize Tree
		tree.setEditable(false);
		tree.getSelectionModel().setSelectionMode(TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
		tree.setShowsRootHandles(true);

		// Initialize Tree-renderer
		renderer.setLeafIcon(new ImageIcon("dmc/leaf.png"));
		tree.setCellRenderer(renderer);

		// Initizlize Scroll Pane
		treeView.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		treeView.setMinimumSize(new Dimension(100, 100));

		tree.addMouseListener(new MouseAdapter(){
			public void mousePressed(MouseEvent e) {
				int selRow = tree.getRowForLocation(e.getX(), e.getY());
				TreePath selPath = tree.getPathForLocation(e.getX(), e.getY());
				if(selRow != -1) {
					DefaultMutableTreeNode sel = (DefaultMutableTreeNode)selPath.getLastPathComponent();
					if(sel.getParent() != null && sel.getParent().getParent() != null && sel.getParent().getParent().getParent() != null)
						;//mTC.openBuddy(sel.toString());
				} 
			}
		});

		return treeView;
	}

	/**
	 * Registers the messaging GUI to allow the buddy list to open
	 *	windows for conversations.
	 * <p>
	 * This may be abstracted away at some point, but for now it
	 * 	is a TabbedConnection.
	 *
	 * @param tc GUI to use to create windows when a buddy is clicked.
	 */
	protected void registerMessageGUI(TabbedConnection tc) {
		mTC = tc;
	}
}
